/*************************************************************************************************************
  SG12232A_Demo.c

  This is a demonstration of the SG12232A and graphics driver running on a PIC16F877A

  Version 1.0   Apr 2009

  Copyright 2009 Geoff Graham - http://geoffg.net
  This program is free software: you can redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation, either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.  You should have received a copy of the GNU General Public License along with this program.
  If not, see <http://www.gnu.org/licenses/>.
	
  Development Environment
	To compile this you need:
	 - CCS C Compiler Version 4.0 or higher (www.ccsinfo.com)
	 - Microchip MPLAB IDE Version 8.0 or higher (www.microchip.com)

  Caveats:
	 - It is written for the CCS C compiler
	 - The 16F877A uses an approx 8MHz clock consisting of a 2.2K resistor and 27pF cap on OSC1 (pin 13)
	 - The 4KHz clock for the LCD is generated by a never ending loop (not practical in the real world)
	
***************************************************************************************************************/

#include <16F877A.h>
#device ADC=8					// read just 8 bits from the ADC
#fuses RC,NOWDT,NOPUT,NOCPD,NOWRT,NOPROTECT,NOBROWNOUT,NOLVP,DEBUG

// Clock Related Fuses (select one):
//        LP  Low-Power Crystal
//        XT  Crystal/Resonator
//        HS  High-Speed Crystal/Resonator
//        RC  External Resistor-Capacitor (RC) with FOSC/4 output on RA6
// Watch Dog Timer:              WDT, NOWDT
// Power-up Timer:               PUT,NOPUT
// Protect Code:                 PROTECT,NOPROTECT
// In-Circuit Debugger Mode:     DEBUG,NODEBUG
// Brown-out Detect:             BROWNOUT,NOBROWNOUT
// Low-Volt Programming Enable:  LVP, NOLVP
// Protect Data Memory:          CPD,NOCPD
// Prog mem write protected      WRT_50%,WRT_5%,WRT_25%,NOWRT

// processor specific hardware registers
#byte porta = 5						// Port A I/O register
#byte portb = 6						// Port A I/O register
#byte portc = 7						// Port B I/O register
#byte tris_a = 0x85					// Port A TRIS register
#byte tris_b = 0x86					// Port B TRIS register
#byte tris_c = 0x87					// Port B TRIS register

#use delay(clock=8000000)

#use fast_io(A)	
#use fast_io(B)
#use fast_io(C)
	
#define TRISA_INIT		0b00000000		// this MUST agree with the #defines below
#BIT	LCD_CS2			= porta.0		// output 	LCD Chip Select 1
#BIT	LCD_CS1			= porta.1		// output 	LCD Chip Select 2
#BIT 	LCD_A0			= porta.2		// output  	LCD A0
#BIT 	LCD_RW			= porta.3		// output 	LCD Read/Write
	
#define TRISB_INIT		0b00000000		// this MUST agree with the #defines below
#BIT 	LCD_CLK			= portb.0		// output 	LCD Clock (approx 4KHz squarewave)
#BIT 	LCD_E			= portb.1		// output 	LCD Enable (data strobe)
	
#BIT 	C0			= portc.0		// input & output - Data bus to LCD
#BIT 	C1			= portc.1		// input & output - Data bus to LCD
#BIT 	C2			= portc.2		// input & output - Data bus to LCD
#BIT 	C3			= portc.3		// input & output - Data bus to LCD
#BIT 	C4			= portc.4		// input & output - Data bus to LCD
#BIT 	C5			= portc.5		// input & output - Data bus to LCD
#BIT 	C6			= portc.6		// input & output - Data bus to LCD
#BIT 	C7			= portc.7		// input & output - Data bus to LCD
	
#define LCD_DATA 		portc			// Data port for the LCD
#define LCD_DATA_TRIS		tris_c			// and its associated direction register	


// define some easy to remember types
#define bit int1
#define int8 signed int8
#define uint8 unsigned int8
#define int16 signed int16
#define uint16 unsigned int16
#define int32 signed int32
#define uint32 unsigned int32

/**********************************************************************************************
Main program
**********************************************************************************************/
#include "./SG12232A_Driver.h"				// this is the LCD driver
#include "./glcd_library.h"				// and this is a universal graphics/fonts package

void main() {
    setup_adc_ports(NO_ANALOGS);
	setup_comparator(NC_NC_NC_NC);
	SETUP_TIMER_0(RTCC_INTERNAL);
	
	// set pin directions
	tris_a = TRISA_INIT;
	tris_b = TRISB_INIT;
	
	LCD_CS2 = LCD_CS1 = LCD_A0 = 1;
	LCD_RW = LCD_E = 0;
	
	GLCD_Init();					// defined in SG12232A_Driver.h
	glcd_setxy(0, 0, 1);				// defined in glcd_library.h
	printf(glcd_putc916, "Hello World!");		// glcd_putc916 is defined in glcd_library.h
	while(1) {
		LCD_CLK = !LCD_CLK;			// generate an approx 4KHz squarewave for the LCD
		delay_us(340);
	}	
}

