//***************************************************************************
//                     glcd_library.c
//
//  This file contains a library of routines for handling a graphics LCD
//
//
//****************************************************************************
//
//   GRAPHICS ROUTINES
//
//   glcd_line(x1,y1,x2,y2,colour)
//      * Draws a line from the first point to the second point
//        with the given colour.
//        - colour can be ON or OFF
//
//   glcd_rect(x1,y1,x2,y2,fill,colour)
//      * Draws a rectangle with upper left point (x1,y1) and lower
//        right point (x2,y2).
//        - fill can be YES or NO
//        - colour can be ON or OFF
//
//   glcd_bar(x1,y1,x2,y2,width,colour)
//      * Draws a bar (wide line) from the first point to the
//        second point.
//        - width is the number of pixels wide
//        - colour is ON or OFF
//
//   glcd_circle(x,y,radius,fill,colour)
//      * Draws a circle with center at (x,y)
//        - fill can be YES or NO
//        - colour can be ON or OFF
//
//******************************************************************************
//
//  TEXT WRITING ROUTINES
//
//  There are 5 fonts:
//
//   - 3x6 font and the associated code occupies approx 850 bytes
//     This is numeric and uppercase only and is very hard to read
//
//   - 5x7 font and the associated code occupies approx 1.2k bytes
//     This is a small but readable font giving 8 lines & 20 char per line
//
//   - 8x12 font and the associated code occupies approx 2K bytes
//     This is good for readability and gives 5 lines
// 	   at 16 characters per line (with size = 1).
// 
//   - 9x16 font and the associated code occupies approx 2.7K bytes
//     This is the best font if you have the space as it is very readable and
//     gives 4 lines at 14 characters per line (with size = 1).
//     
//   - 11x16 font and the associated code occupies approx 3.5K bytes
//     This is also very readable and gives 4 lines at 10 characters per line
//     (with size = 1).
//
//   Any or all together will be linked by calling the appropriate functions.
// 
//
//   Before calling ANY text function the starting position and optionally other
//   characteristics must be set by calling glcd_setxy()
//
//   void glcd_setxy(x, y, size, colour)
//       x and y: The upper left coordinate of the first character
//       size:    An integer that scales the size of the text (default is 1)
//       colour:  ON or OFF (default is ON)
//
//   If you are happy with the defaults you only need to specify x & y, for example:
//       glcd_setxy(0, 0);			// top left hand corner
//
//   All text functions update x and y automatically so that the invisible cursor is
//   ready for the next character.  The character \r will return the cursor to the
//   left margin and \n will cause the y position to step down a line.
//
//   Note that character wrapping is not supported, strings are truncated at the
//   edge of the display.
//
//   The data is stored in global variables and, as a shortcut, these can be accessed
//   directly.  The variables are:
//   	int8 glcd_x, glcd_y			// Coordinates
//		int8 glcd_size				// Size
//		int1 glcd_colour			// Colour
//
//   The writing functions are:
//
//   glcd_putc36(char)
//      * Write char in 3x6 font
//
//   glcd_text36(textptr)
//      * Write the null terminated text pointed to by textptr in 3x6 font
//
//   glcd_putc57(char)
//      * Write char in 5x7 font
//
//   glcd_text57(textptr)
//      * Write the null terminated text pointed to by textptr in 5x7 font
//
//   glcd_putc812(char)
//      * Write char in 8x12 font
//      * Note that the tild character is replaced with a degrees C symbol
//
//   glcd_text812(textptr)
//      * Write the null terminated text pointed to by textptr in 8x12 font
//
//   glcd_putc916(char)
//      * Write char in 9x16 font
//      * Note that the tild character is replaced with a degrees C symbol
//
//   glcd_text916(textptr)
//      * Write the null terminated text pointed to by textptr in 9x16 font
//
//   glcd_putc1116(char)
//      * Write char in 11x16 font
//      * Note that the tild character is replaced with a degrees C symbol
//
//   glcd_text1116(textptr)
//      * Write the null terminated text pointed to by textptr in 11x16 font
//
//   The glcd_putcXX(char) functions can be used in printf().  For example:
//      printf(glcd_char812, "Temperature is %d~", temperature);
//      * Note that the tild (~) character is replaced with a degrees C symbol
//        in the 8x12 font
// 
//
//**********************************************************************************
//    LCD DRIVER
//
//   The above routines are dependent on a graphics driver that exposes the following
//   function:
//
//   glcd_pixel(x,y,colour)
//      * Sets the pixel to the given colour.
//        - colour can be ON or OFF
//
//
//   Other functions that the driver should supply (but are not used by this library) are:
//
//   glcd_init()
//		* Initilise the display
//      * Must be called before any other function.
//
//   glcd_fillScreen(colour)
//      * Fills the entire LCD with the given colour.
//        - colour can be ON or OFF
//
//
//**********************************************************************************

byte glcd_x, glcd_y;					// Coordinates used to specify where the next text will go
byte glcd_size;							// Same for size
bit glcd_colour;						// Same for colour

#inline
void glcd_setxy(x, y, sz = 1, cl = 1) {
	 glcd_x = x; glcd_y = y; 			// set the coordinates
	 glcd_size = sz; glcd_colour = cl;	// set the other factors
}

byte glcd_readByte(byte chip);
void glcd_writeByte(byte chip, byte data);
void glcd_fillScreen(int1 colour);


const BYTE TEXT_3x6_1[0x41][3] ={0x00,0x00,0x00, // Space	0x20
						0x00,0x5C,0x00, // !     
						0x0C,0x00,0x0C, // "
						0x7C,0x28,0x7C, // #
						0x7C,0x44,0x7C, // 0x                         
						0x24,0x10,0x48, // %
						0x28,0x54,0x08, // &
						0x00,0x0C,0x00, // '                          
						0x38,0x44,0x00, // (                          
						0x44,0x38,0x00, // )                          
						0x20,0x10,0x08, // //                         
						0x10,0x38,0x10, // +                          
						0x80,0x40,0x00, // ,                          
						0x10,0x10,0x10, // -                          
						0x00,0x40,0x00, // .                          
						0x20,0x10,0x08, // /     
						0x38,0x44,0x38, // 0	0x30                      
						0x00,0x7C,0x00, // 1                          
						0x64,0x54,0x48, // 2                          
						0x44,0x54,0x28, // 3                          
						0x1C,0x10,0x7C, // 4                          
						0x4C,0x54,0x24, // 5                          
						0x38,0x54,0x20, // 6                          
						0x04,0x74,0x0C, // 7                          
						0x28,0x54,0x28, // 8                          
						0x08,0x54,0x38, // 9                          
						0x00,0x50,0x00, // :                          
						0x80,0x50,0x00, // ;                          
						0x10,0x28,0x44, // <                          
						0x28,0x28,0x28, // =                   
						0x44,0x28,0x10, // >                         
						0x04,0x54,0x08, // ?                          
						0x38,0x4C,0x5C, // @	0x40                           
						0x78,0x14,0x78, // A                          
						0x7C,0x54,0x28, // B                          
						0x38,0x44,0x44, // C                          
						0x7C,0x44,0x38, // D                          
						0x7C,0x54,0x44, // E                          
						0x7C,0x14,0x04, // F                          
						0x38,0x44,0x34, // G                          
						0x7C,0x10,0x7C, // H                          
						0x00,0x7C,0x00, // I                          
						0x20,0x40,0x3C, // J                          
						0x7C,0x10,0x6C, // K                          
						0x7C,0x40,0x40, // L                          
						0x7C,0x08,0x7C, // M                          
						0x7C,0x04,0x7C, // N                          
						0x7C,0x44,0x7C, // O                          
						0x7C,0x14,0x08, // P	0x50                  
						0x38,0x44,0x78, // Q                          
						0x7C,0x14,0x68, // R                          
						0x48,0x54,0x24, // S                          
						0x04,0x7C,0x04, // T                          
						0x7C,0x40,0x7C, // U                          
						0x3C,0x40,0x3C, // V                          
						0x7C,0x20,0x7C, // W                          
						0x6C,0x10,0x6C, // X                          
						0x1C,0x60,0x1C, // Y                          
						0x64,0x54,0x4C, // Z                          
						0x7C,0x44,0x00, // [                          
						0x08,0x10,0x20, // \                          
						0x44,0x7C,0x00, // ]                          
						0x08,0x04,0x08, // ^                          
						0x80,0x80,0x80, // _                          
						0x04,0x08,0x00 // `	0x60                  
};


const BYTE TEXT_5x7_1[51][5] ={0x00, 0x00, 0x00, 0x00, 0x00, // SPACE
                         0x00, 0x00, 0x5F, 0x00, 0x00, // !
                         0x00, 0x03, 0x00, 0x03, 0x00, // "
                         0x14, 0x3E, 0x14, 0x3E, 0x14, // #
                         0x24, 0x2A, 0x7F, 0x2A, 0x12, // $
                         0x43, 0x33, 0x08, 0x66, 0x61, // %
                         0x36, 0x49, 0x55, 0x22, 0x50, // &
                         0x00, 0x05, 0x03, 0x00, 0x00, // '
                         0x00, 0x1C, 0x22, 0x41, 0x00, // (
                         0x00, 0x41, 0x22, 0x1C, 0x00, // )
                         0x14, 0x08, 0x3E, 0x08, 0x14, // *
                         0x08, 0x08, 0x3E, 0x08, 0x08, // +
                         0x00, 0x50, 0x30, 0x00, 0x00, // ,
                         0x08, 0x08, 0x08, 0x08, 0x08, // -
                         0x00, 0x60, 0x60, 0x00, 0x00, // .
                         0x20, 0x10, 0x08, 0x04, 0x02, // /
                         0x3E, 0x51, 0x49, 0x45, 0x3E, // 0
                         0x04, 0x02, 0x7F, 0x00, 0x00, // 1
                         0x42, 0x61, 0x51, 0x49, 0x46, // 2
                         0x22, 0x41, 0x49, 0x49, 0x36, // 3
                         0x18, 0x14, 0x12, 0x7F, 0x10, // 4
                         0x27, 0x45, 0x45, 0x45, 0x39, // 5
                         0x3E, 0x49, 0x49, 0x49, 0x32, // 6
                         0x01, 0x01, 0x71, 0x09, 0x07, // 7
                         0x36, 0x49, 0x49, 0x49, 0x36, // 8
                         0x26, 0x49, 0x49, 0x49, 0x3E, // 9
                         0x00, 0x36, 0x36, 0x00, 0x00, // :
                         0x00, 0x56, 0x36, 0x00, 0x00, // ;
                         0x08, 0x14, 0x22, 0x41, 0x00, // <
                         0x14, 0x14, 0x14, 0x14, 0x14, // =
                         0x00, 0x41, 0x22, 0x14, 0x08, // >
                         0x02, 0x01, 0x51, 0x09, 0x06, // ?
                         0x3E, 0x41, 0x59, 0x55, 0x5E, // @
                         0x7E, 0x09, 0x09, 0x09, 0x7E, // A
                         0x7F, 0x49, 0x49, 0x49, 0x36, // B
                         0x3E, 0x41, 0x41, 0x41, 0x22, // C
                         0x7F, 0x41, 0x41, 0x41, 0x3E, // D
                         0x7F, 0x49, 0x49, 0x49, 0x41, // E
                         0x7F, 0x09, 0x09, 0x09, 0x01, // F
                         0x3E, 0x41, 0x41, 0x49, 0x3A, // G
                         0x7F, 0x08, 0x08, 0x08, 0x7F, // H
                         0x00, 0x41, 0x7F, 0x41, 0x00, // I
                         0x30, 0x40, 0x40, 0x40, 0x3F, // J
                         0x7F, 0x08, 0x14, 0x22, 0x41, // K
                         0x7F, 0x40, 0x40, 0x40, 0x40, // L
                         0x7F, 0x02, 0x0C, 0x02, 0x7F, // M
                         0x7F, 0x02, 0x04, 0x08, 0x7F, // N
                         0x3E, 0x41, 0x41, 0x41, 0x3E, // O
                         0x7F, 0x09, 0x09, 0x09, 0x06, // P
                         0x1E, 0x21, 0x21, 0x21, 0x5E, // Q
                         0x7F, 0x09, 0x09, 0x09, 0x76};// R

const BYTE TEXT_5x7_2[44][5]={0x26, 0x49, 0x49, 0x49, 0x32, // S
                         0x01, 0x01, 0x7F, 0x01, 0x01, // T
                         0x3F, 0x40, 0x40, 0x40, 0x3F, // U
                         0x1F, 0x20, 0x40, 0x20, 0x1F, // V
                         0x7F, 0x20, 0x10, 0x20, 0x7F, // W
                         0x41, 0x22, 0x1C, 0x22, 0x41, // X
                         0x07, 0x08, 0x70, 0x08, 0x07, // Y
                         0x61, 0x51, 0x49, 0x45, 0x43, // Z
                         0x00, 0x7F, 0x41, 0x00, 0x00, // [
                         0x02, 0x04, 0x08, 0x10, 0x20, // \
                         0x00, 0x00, 0x41, 0x7F, 0x00, // ]
                         0x04, 0x02, 0x01, 0x02, 0x04, // ^
                         0x40, 0x40, 0x40, 0x40, 0x40, // _
                         0x00, 0x01, 0x02, 0x04, 0x00, // `
                         0x20, 0x54, 0x54, 0x54, 0x78, // a
                         0x7F, 0x44, 0x44, 0x44, 0x38, // b
                         0x38, 0x44, 0x44, 0x44, 0x44, // c
                         0x38, 0x44, 0x44, 0x44, 0x7F, // d
                         0x38, 0x54, 0x54, 0x54, 0x18, // e
                         0x04, 0x04, 0x7E, 0x05, 0x05, // f
                         0x08, 0x54, 0x54, 0x54, 0x3C, // g
                         0x7F, 0x08, 0x04, 0x04, 0x78, // h
                         0x00, 0x44, 0x7D, 0x40, 0x00, // i
                         0x20, 0x40, 0x44, 0x3D, 0x00, // j
                         0x7F, 0x10, 0x28, 0x44, 0x00, // k
                         0x00, 0x41, 0x7F, 0x40, 0x00, // l
                         0x7C, 0x04, 0x78, 0x04, 0x78, // m
                         0x7C, 0x08, 0x04, 0x04, 0x78, // n
                         0x38, 0x44, 0x44, 0x44, 0x38, // o
                         0x7C, 0x14, 0x14, 0x14, 0x08, // p
                         0x08, 0x14, 0x14, 0x14, 0x7C, // q
                         0x00, 0x7C, 0x08, 0x04, 0x04, // r
                         0x48, 0x54, 0x54, 0x54, 0x20, // s
                         0x04, 0x04, 0x3F, 0x44, 0x44, // t
                         0x3C, 0x40, 0x40, 0x20, 0x7C, // u
                         0x1C, 0x20, 0x40, 0x20, 0x1C, // v
                         0x3C, 0x40, 0x30, 0x40, 0x3C, // w
                         0x44, 0x28, 0x10, 0x28, 0x44, // x
                         0x0C, 0x50, 0x50, 0x50, 0x3C, // y
                         0x44, 0x64, 0x54, 0x4C, 0x44, // z
                         0x00, 0x08, 0x36, 0x41, 0x41, // {
                         0x00, 0x00, 0x7F, 0x00, 0x00, // |
                         0x41, 0x41, 0x36, 0x08, 0x00, // }
                         0x02, 0x01, 0x02, 0x04, 0x02};// ~

	
const BYTE TEXT_8x12_1[21][12]={ 0,0,0,0,0,0,0,0,0,0,0,0                // ' ' 32
						0,24,60,60,60,24,24,0,24,24,0,0,                // '!' 33
						54,54,54,20,0,0,0,0,0,0,0,0,                    // '"' 34
						0,108,108,108,254,108,108,254,108,108,0,0,      // '#' 35
						24,24,124,198,192,120,60,6,198,124,24,24,       // '$' 36
						0,0,0,98,102,12,24,48,102,198,0,0,              // '%' 37
						0,56,108,56,56,118,246,206,204,118,0,0,         // '&' 38
						12,12,12,24,0,0,0,0,0,0,0,0,                    // ''' 39
						0,12,24,48,48,48,48,48,24,12,0,0,               // '(' 40
						0,48,24,12,12,12,12,12,24,48,0,0,               // ')' 41
						0,0,0,108,56,254,56,108,0,0,0,0,                // '*' 42
						0,0,0,24,24,126,24,24,0,0,0,0,                  // '+' 43
						0,0,0,0,0,0,0,12,12,12,24,0,                    // ',' 44
						0,0,0,0,0,254,0,0,0,0,0,0,                      // '-' 45
						0,0,0,0,0,0,0,0,24,24,0,0,                      // '.' 46
						0,0,2,6,12,24,48,96,192,128,0,0,                // '/' 47
						0,124,198,206,222,246,230,198,198,124,0,0,      // '0' 48
						0,24,120,24,24,24,24,24,24,126,0,0,             // '1' 49
						0,124,198,198,12,24,48,96,198,254,0,0,          // '2' 50
						0,124,198,6,6,60,6,6,198,124,0,0,               // '3' 51
						0,12,28,60,108,204,254,12,12,12,0,0             // '4' 52
};
						
const BYTE TEXT_8x12_2[21][12]={ 0,254,192,192,192,252,6,6,198,124,0,0, // '5' 53
						0,124,198,192,192,252,198,198,198,124,0,0,      // '6' 54
						0,254,198,12,24,48,48,48,48,48,0,0,             // '7' 55
						0,124,198,198,198,124,198,198,198,124,0,0,      // '8' 56
						0,124,198,198,198,126,6,6,198,124,0,0,          // '9' 57
						0,0,0,12,12,0,0,12,12,0,0,0,                    // ':' 58
						0,0,0,12,12,0,0,12,12,12,24,0,                  // ';' 59
						0,12,24,48,96,192,96,48,24,12,0,0,              // '<' 60
						0,0,0,0,254,0,254,0,0,0,0,0,                    // '=' 61
						0,96,48,24,12,6,12,24,48,96,0,0,                // '>' 62
						0,124,198,198,12,24,24,0,24,24,0,0,             // '?' 63
						0,124,198,198,222,222,222,220,192,126,0,0,      // '@' 64
						0,56,108,198,198,198,254,198,198,198,0,0,       // 'A' 65
						0,252,102,102,102,124,102,102,102,252,0,0,      // 'B' 66
						0,60,102,192,192,192,192,192,102,60,0,0,        // 'C' 67
						0,248,108,102,102,102,102,102,108,248,0,0,      // 'D' 68
						0,254,102,96,96,124,96,96,102,254,0,0,          // 'E' 69
						0,254,102,96,96,124,96,96,96,240,0,0,           // 'F' 70
						0,124,198,198,192,192,206,198,198,124,0,0,      // 'G' 71
						0,198,198,198,198,254,198,198,198,198,0,0,      // 'H' 72
						0,60,24,24,24,24,24,24,24,60,0,0                // 'I' 73
};
						
const BYTE TEXT_8x12_3[21][12]={ 0,60,24,24,24,24,24,216,216,112,0,0,   // 'J' 74
						0,198,204,216,240,240,216,204,198,198,0,0,      // 'K' 75
						0,240,96,96,96,96,96,98,102,254,0,0,            // 'L' 76
						0,198,198,238,254,214,214,214,198,198,0,0,      // 'M' 77
						0,198,198,230,230,246,222,206,206,198,0,0,      // 'N' 78
						0,124,198,198,198,198,198,198,198,124,0,0,      // 'O' 79
						0,252,102,102,102,124,96,96,96,240,0,0,         // 'P' 80
						0,124,198,198,198,198,198,198,214,124,6,0,      // 'Q' 81
						0,252,102,102,102,124,120,108,102,230,0,0,      // 'R' 82
						0,124,198,192,96,56,12,6,198,124,0,0,           // 'S' 83
						0,126,90,24,24,24,24,24,24,60,0,0,              // 'T' 84
						0,198,198,198,198,198,198,198,198,124,0,0,      // 'U' 85
						0,198,198,198,198,198,198,108,56,16,0,0,        // 'V' 86
						0,198,198,214,214,214,254,238,198,198,0,0,      // 'W' 87
						0,198,198,108,56,56,56,108,198,198,0,0,         // 'X' 88
						0,102,102,102,102,60,24,24,24,60,0,0,           // 'Y' 89
						0,254,198,140,24,48,96,194,198,254,0,0,         // 'Z' 90
						0,124,96,96,96,96,96,96,96,124,0,0,             // '[' 91
						0,0,128,192,96,48,24,12,6,2,0,0,                // '\' 92
						0,124,12,12,12,12,12,12,12,124,0,0,             // ']' 93
						16,56,108,198,0,0,0,0,0,0,0,0                   // '^' 94
};
						
const BYTE TEXT_8x12_4[21][12]={ 0,0,0,0,0,0,0,0,0,0,0,255,             // '_' 95
						24,24,24,12,0,0,0,0,0,0,0,0,                    // '`' 96
						0,0,0,0,120,12,124,204,220,118,0,0,             // 'a' 97
						0,224,96,96,124,102,102,102,102,252,0,0,        // 'b' 98
						0,0,0,0,124,198,192,192,198,124,0,0,            // 'c' 99
						0,28,12,12,124,204,204,204,204,126,0,0,         // 'd' 100
						0,0,0,0,124,198,254,192,198,124,0,0,            // 'e' 101
						0,28,54,48,48,252,48,48,48,120,0,0,             // 'f' 102
						0,0,0,0,118,206,198,198,126,6,198,124,          // 'g' 103
						0,224,96,96,108,118,102,102,102,230,0,0,        // 'h' 104
						0,24,24,0,56,24,24,24,24,60,0,0,                // 'i' 105
						0,12,12,0,28,12,12,12,12,204,204,120,           // 'j' 106
						0,224,96,96,102,108,120,108,102,230,0,0,        // 'k' 107
						0,56,24,24,24,24,24,24,24,60,0,0,               // 'l' 108
						0,0,0,0,108,254,214,214,198,198,0,0,            // 'm' 109
						0,0,0,0,220,102,102,102,102,102,0,0,            // 'n' 110
						0,0,0,0,124,198,198,198,198,124,0,0,            // 'o' 111
						0,0,0,0,220,102,102,102,124,96,96,240,          // 'p' 112
						0,0,0,0,118,204,204,204,124,12,12,30,           // 'q' 113
						0,0,0,0,220,102,96,96,96,240,0,0,               // 'r' 114
						0,0,0,0,124,198,112,28,198,124,0,0              // 's' 115
};
						
const BYTE TEXT_8x12_5[11][12]={ 0,48,48,48,252,48,48,48,54,28,0,0,     // 't' 116
						0,0,0,0,204,204,204,204,204,118,0,0,            // 'u' 117
						0,0,0,0,198,198,198,108,56,16,0,0,              // 'v' 118
						0,0,0,0,198,198,214,214,254,108,0,0,            // 'w' 119
						0,0,0,0,198,108,56,56,108,198,0,0,              // 'x' 120
						0,0,0,0,198,198,198,206,118,6,198,124,          // 'y' 121
						0,0,0,0,254,140,24,48,98,254,0,0,               // 'z' 122
						0,14,24,24,24,112,24,24,24,14,0,0,              // '{' 123
						0,24,24,24,24,0,24,24,24,24,0,0,                // '|' 124
						0,112,24,24,24,14,24,24,24,112,0,0,             // '}' 125
						0,70,169,168,72,8,9,6,0,0,0,0                   // Degree C symbol (oC) 126
//						0,118,220,0,0,0,0,0,0,0,0,0,                    // '~' 126
};


const unsigned int16 TEXT_9x16_1[14][9]={ 
        0, 0, 0, 0, 0, 0, 0, 0, 0,                                        //  
        0, 0, 62, 13311, 13311, 62, 0, 0, 0,                              // !
        0, 7, 7, 0, 0, 7, 7, 0, 0,                                        // "
        288, 8190, 8190, 288, 288, 8190, 8190, 288, 0,                    // #
        1592, 3644, 2148, 16383, 16383, 2436, 3868, 1816, 0,              // $
        12318, 7219, 1822, 448, 112, 7708, 13063, 7681, 0,                // %
        7966, 12723, 8417, 8417, 12723, 7966, 6912, 12672, 0,             // &
        0, 0, 0, 11, 7, 0, 0, 0, 0,                                       // '
        0, 0, 2032, 16382, 30735, 16385, 0, 0, 0,                         // (
        0, 0, 16385, 30735, 16382, 2032, 0, 0, 0,                         // )
        128, 672, 992, 448, 448, 992, 672, 128, 0,                        // *
        128, 128, 128, 992, 992, 128, 128, 128, 0,                        // +
        0, 0, 0, 45056, 28672, 0, 0, 0, 0,                                // ,
        128, 128, 128, 128, 128, 128, 128, 128, 0,                        // -
};

const unsigned int16 TEXT_9x16_2[14][9]={ 
        0, 0, 0, 12288, 12288, 0, 0, 0, 0,                                // .
        12288, 15360, 3840, 960, 240, 60, 15, 3, 0,                       // /
        4092, 8190, 13827, 8577, 8289, 12315, 8190, 4092, 0,              // 0
        8196, 8196, 8198, 16383, 16383, 8192, 8192, 8192, 0,              // 1
        14348, 15374, 9731, 8961, 8577, 8387, 8318, 8252, 0,              // 2
        3084, 7182, 12355, 8257, 8257, 12355, 8190, 4028, 0,              // 3
        3840, 4064, 2300, 2079, 16259, 16256, 2048, 2048, 0,              // 4
        3135, 7231, 12321, 8225, 8225, 12385, 8129, 3969, 0,              // 5
        4064, 8184, 12380, 8262, 8259, 12481, 8065, 3841, 0,              // 6
        1, 1, 1, 15361, 16257, 1009, 127, 15, 0,                          // 7
        3868, 8126, 12515, 8257, 8257, 12515, 8126, 3868, 0,              // 8
        8252, 8318, 8387, 12417, 6273, 3715, 2046, 508, 0,                // 9
        0, 0, 0, 12384, 12384, 0, 0, 0, 0,                                // :
        0, 0, 0, 45152, 28768, 0, 0, 0, 0,                                // ;
} ;

const unsigned int16 TEXT_9x16_3[14][9]={ 
        128, 448, 864, 1584, 3096, 6156, 4100, 0, 0,                      // <
        288, 288, 288, 288, 288, 288, 288, 288, 0,                        // =
        4100, 6156, 3096, 1584, 864, 448, 128, 0, 0,                      // >
        6, 7, 3, 13057, 13249, 243, 63, 30, 0,                            // ?
        4092, 8190, 12291, 8673, 9201, 8721, 8467, 13310, 5116,           // @
        16380, 16382, 259, 257, 257, 259, 16382, 16380, 0,                // A
        16383, 16383, 8257, 8257, 8257, 12515, 8126, 3868, 0,             // B
        4092, 8190, 12291, 8193, 8193, 12291, 7182, 3084, 0,              // C
        16383, 16383, 8193, 8193, 8193, 14343, 8190, 2040, 0,             // D
        16383, 16383, 8257, 8257, 8257, 8257, 8257, 8193, 0,              // E
        16383, 16383, 129, 129, 129, 129, 129, 1, 0,                      // F
        4092, 8190, 12291, 8193, 8321, 12419, 8078, 3980, 0,              // G
        16383, 16383, 64, 64, 64, 64, 16383, 16383, 0,                    // H
        8193, 8193, 8193, 16383, 16383, 8193, 8193, 8193, 0,              // I       
} ;

const unsigned int16 TEXT_9x16_4[14][9]={ 
        3072, 7168, 12288, 8192, 8192, 12288, 8191, 4095, 0,              // J
        16383, 16383, 192, 480, 1848, 3612, 15375, 12291, 0,              // K
        16383, 16383, 8192, 8192, 8192, 8192, 8192, 8192, 0,              // L
        16383, 16383, 120, 960, 960, 120, 16383, 16383, 0,                // M
        16383, 16383, 28, 112, 448, 1792, 16383, 16383, 0,                // N
        4092, 8190, 12291, 8193, 8193, 12291, 8190, 4092, 0,              // O
        16383, 16383, 129, 129, 129, 195, 126, 60, 0,                     // P
        4092, 8190, 12291, 8193, 24577, 61443, 40958, 4092, 0,            // Q
        16383, 16383, 129, 129, 385, 8131, 15998, 8252, 0,                // R
        3132, 7294, 12387, 8417, 8641, 12675, 8078, 3852, 0,              // S
        1, 1, 1, 16383, 16383, 1, 1, 1, 0,                                // T
        4095, 8191, 12288, 8192, 8192, 12288, 8191, 4095, 0,              // U
        63, 511, 4032, 15360, 15360, 4032, 511, 63, 0,                    // V
        511, 16383, 15872, 448, 448, 15872, 16383, 511, 0,                // W
} ;

const unsigned int16 TEXT_9x16_5[14][9]={ 
        15375, 16191, 1008, 192, 192, 1008, 16191, 15375, 0,              // X
        63, 255, 448, 16256, 16256, 448, 255, 63, 0,                      // Y
        14337, 15873, 10113, 8641, 8289, 8249, 8223, 8199, 0,             // Z
        0, 0, 16383, 16383, 8193, 8193, 0, 0, 0,                          // [
        3, 15, 60, 240, 960, 3840, 15360, 12288, 0,                       // \
        0, 0, 8193, 8193, 16383, 16383, 0, 0, 0,                          // ]
        4, 4, 6, 3, 3, 6, 6, 4, 0,                                        // ^
        32768, 32768, 32768, 32768, 32768, 32768, 32768, 32768, 32768,    // _
        0, 0, 0, 7, 11, 0, 0, 0, 0,                                       // `
        7680, 16192, 8480, 8480, 8480, 4384, 16352, 16320, 0,             // a
        16383, 16383, 4160, 8224, 8224, 12384, 8128, 3968, 0,             // b
        3968, 8128, 12384, 8224, 8224, 8224, 12384, 4160, 0,              // c
        3968, 8128, 12384, 8224, 8224, 4160, 16383, 16383, 0,             // d
        3968, 8128, 12896, 8736, 8736, 8800, 13248, 4992, 0,              // e
} ;

const unsigned int16 TEXT_9x16_6[14][9]={ 
        32, 32, 16382, 16383, 33, 33, 35, 2, 0,                           // f
        18304, 20416, 55392, 36896, 36896, 51264, 32736, 16352, 0,        // g
        16383, 16383, 64, 32, 32, 96, 16320, 16256, 0,                    // h
        0, 8224, 8224, 16355, 16355, 8192, 8192, 0, 0,                    // i
        0, 16384, 49184, 32800, 65507, 32739, 0, 0, 0,                    // j
        16383, 16383, 1536, 1792, 3456, 6336, 12384, 8224, 0,             // k
        0, 8193, 8193, 16383, 16383, 8192, 8192, 0, 0,                    // l
        16352, 16352, 96, 8128, 8128, 96, 16352, 16320, 0,                // m
        16352, 16352, 64, 32, 32, 96, 16320, 16256, 0,                    // n
        3968, 8128, 12384, 8224, 8224, 12384, 8128, 3968, 0,              // o
        65504, 65504, 4160, 8224, 8224, 12384, 8128, 3968, 0,             // p
        3968, 8128, 8224, 8224, 8224, 4160, 65504, 65504, 0,              // q
        16352, 16352, 192, 64, 96, 96, 96, 64, 0,                         // r
        4288, 12768, 8480, 8992, 9760, 9248, 15456, 6208, 0,              // s
} ;

const unsigned int16 TEXT_9x16_7[11][9]={ 
        0, 32, 32, 8190, 16382, 8224, 8224, 0, 0,                         // t
        8160, 16352, 8192, 8192, 8192, 4096, 16352, 16352, 0,             // u
        992, 2016, 7168, 12288, 12288, 7168, 2016, 992, 0,                // v
        2016, 16352, 14336, 1984, 1984, 14336, 16352, 2016, 0,            // w
        12384, 15840, 3456, 1792, 1792, 3456, 15840, 12384, 0,            // x
        33760, 36832, 56320, 28672, 12288, 7168, 4064, 992, 0,            // y
        12320, 14368, 11296, 9760, 8992, 8608, 8416, 8288, 0,             // z
        0, 0, 128, 16382, 32639, 16385, 16385, 0, 0,                      // {
        0, 0, 0, 65407, 65407, 0, 0, 0, 0,                                // |
        0, 16385, 16385, 32639, 16382, 128, 0, 0, 0,                      // }
        2, 2, 1, 1, 2, 2, 1, 1, 0,                                        // ~
};


const unsigned int16 TEXT_11x16_1[11][11]={
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                                                //   
        0, 0, 0, 124, 13311, 13311, 124, 0, 0, 0, 0,                                    //  !
        0, 0, 60, 60, 0, 0, 60, 60, 0, 0, 0,                                            //  "
        512, 7696, 8080, 1008, 638, 7710, 8080, 1008, 638, 30, 16,                      //  #
        0, 1144, 3324, 3276, 16383, 16383, 3276, 4044, 1928, 0, 0,                      //  $
        12288, 14392, 7224, 3640, 1792, 896, 448, 14560, 14448, 14392, 28,              //  %
        0, 7936, 16312, 12796, 8646, 14306, 7742, 7196, 13824, 8704, 0,                 //  &
        0, 0, 0, 39, 63, 31, 0, 0, 0, 0, 0,                                             //  '
        0, 0, 1008, 4092, 8190, 14343, 8193, 8193, 0, 0, 0,                             //  (
        0, 0, 8193, 8193, 14343, 8190, 4092, 1008, 0, 0, 0,                             //  )
        0, 3224, 3768, 992, 4088, 4088, 992, 3768, 3224, 0, 0,                          //  *
};

const unsigned int16 TEXT_11x16_2[11][11]={
        0, 384, 384, 384, 4080, 4080, 384, 384, 384, 0, 0                               //  +
        0, 0, 0, 47104, 63488, 30720, 0, 0, 0, 0, 0,                                    //  ,
        0, 384, 384, 384, 384, 384, 384, 384, 384, 0, 0,                                //  -
        0, 0, 0, 14336, 14336, 14336, 0, 0, 0, 0, 0,                                    //  .
        6144, 7168, 3584, 1792, 896, 448, 224, 112, 56, 28, 14,                         //  /
        2040, 8190, 7686, 13059, 12675, 12483, 12387, 12339, 6174, 8190, 2040,          //  0
        0, 0, 12300, 12300, 12302, 16383, 16383, 12288, 12288, 12288, 0,                //  1
        12316, 14366, 15367, 15875, 14083, 13187, 12739, 12515, 12407, 12350, 12316,    //  2
        3084, 7182, 14343, 12483, 12483, 12483, 12483, 12483, 14823, 8062, 3644,        //  3
        960, 992, 880, 824, 796, 782, 775, 16383, 16383, 768, 768,                      //  4
        3135, 7295, 14435, 12387, 12387, 12387, 12387, 12387, 14563, 8131, 3971         //  5
};

const unsigned int16 TEXT_11x16_3[11][11]={
        4032, 8176, 14840, 12508, 12494, 12487, 12483, 12483, 14787, 8064, 3840,        //  6
        3, 3, 3, 12291, 15363, 3843, 963, 243, 63, 15, 3,                               //  7
        3840, 8124, 14846, 12519, 12483, 12483, 12483, 12519, 14846, 8124, 3840,        //  8
        60, 126, 12519, 12483, 12483, 14531, 7363, 3779, 2023, 1022, 252,               //  9
        0, 0, 0, 7280, 7280, 7280, 0, 0, 0, 0, 0,                                       //  :
        0, 0, 0, 40048, 64624, 31856, 0, 0, 0, 0, 0,                                    //  ;
        0, 192, 480, 1008, 1848, 3612, 7182, 14343, 12291, 0, 0,                        //  <
        0, 1632, 1632, 1632, 1632, 1632, 1632, 1632, 1632, 1632, 0,                     //  =
        0, 12291, 14343, 7182, 3612, 1848, 1008, 480, 192, 0, 0,                        //  >
        28, 30, 7, 3, 14211, 14275, 227, 119, 62, 28, 0,                                //  ?
        4088, 8190, 6151, 13299, 14331, 13851, 14331, 14331, 13831, 1022, 504           //  @
};

const unsigned int16 TEXT_11x16_4[11][11]={
        14336, 16128, 2016, 1788, 1567, 1567, 1788, 2016, 16128, 14336, 0,              //  A
        16383, 16383, 12483, 12483, 12483, 12483, 12519, 14846, 8124, 3840, 0,          //  B
        1008, 4092, 7182, 14343, 12291, 12291, 12291, 14343, 7182, 3084, 0,             //  C
        16383, 16383, 12291, 12291, 12291, 12291, 14343, 7182, 4092, 1008, 0,           //  D
        16383, 16383, 12483, 12483, 12483, 12483, 12483, 12483, 12291, 12291, 0,        //  E
        16383, 16383, 195, 195, 195, 195, 195, 195, 3, 3, 0,                            //  F
        1008, 4092, 7182, 14343, 12291, 12483, 12483, 12483, 16327, 16326, 0,           //  G
        16383, 16383, 192, 192, 192, 192, 192, 192, 16383, 16383, 0,                    //  H
        0, 0, 12291, 12291, 16383, 16383, 12291, 12291, 0, 0, 0,                        //  I
        3584, 7680, 14336, 12288, 12288, 12288, 12288, 14336, 8191, 2047, 0,            //  J
        16383, 16383, 192, 480, 1008, 1848, 3612, 7182, 14343, 12291, 0                 //  K
};

const unsigned int16 TEXT_11x16_5[11][11]={
        16383, 16383, 12288, 12288, 12288, 12288, 12288, 12288, 12288, 12288, 0,        //  L
        16383, 16383, 30, 120, 480, 480, 120, 30, 16383, 16383, 0,                      //  M
        16383, 16383, 14, 56, 240, 960, 1792, 7168, 16383, 16383, 0,                    //  N
        1008, 4092, 7182, 14343, 12291, 12291, 14343, 7182, 4092, 1008, 0,              //  O
        16383, 16383, 387, 387, 387, 387, 387, 455, 254, 124, 0,                        //  P
        1008, 4092, 7182, 14343, 12291, 13827, 15879, 7182, 16380, 13296, 0,            //  Q
        16383, 16383, 387, 387, 899, 1923, 3971, 7623, 14590, 12412, 0,                 //  R
        3132, 7294, 14567, 12483, 12483, 12483, 12483, 14791, 8078, 3852, 0,            //  S
        0, 3, 3, 3, 16383, 16383, 3, 3, 3, 0, 0,                                        //  T
        2047, 8191, 14336, 12288, 12288, 12288, 12288, 14336, 8191, 2047, 0,            //  U
        7, 63, 504, 4032, 15872, 15872, 4032, 504, 63, 7, 0                             //  V
};

const unsigned int16 TEXT_11x16_6[11][11]={
        16383, 16383, 7168, 1536, 896, 896, 1536, 7168, 16383, 16383, 0,                //  W
        12291, 15375, 3612, 816, 480, 480, 816, 3612, 15375, 12291, 0,                  //  X
        3, 15, 60, 240, 16320, 16320, 240, 60, 15, 3, 0,                                //  Y
        12291, 15363, 15875, 13059, 12739, 12515, 12339, 12319, 12303, 12291, 0,        //  Z
        0, 0, 16383, 16383, 12291, 12291, 12291, 12291, 0, 0, 0,                        //  [
        14, 28, 56, 112, 224, 448, 896, 1792, 3584, 7168, 6144,                         //  \
        0, 0, 12291, 12291, 12291, 12291, 16383, 16383, 0, 0, 0,                        //  ]
        96, 112, 56, 28, 14, 7, 14, 28, 56, 112, 96,                                    //  ^
        49152, 49152, 49152, 49152, 49152, 49152, 49152, 49152, 49152, 49152, 49152,    // _
        0, 0, 0, 0, 62, 126, 78, 0, 0, 0, 0,                                            //  `
        7168, 15936, 13152, 13152, 13152, 13152, 13152, 13152, 16352, 16320, 0          //  a
};

const unsigned int16 TEXT_11x16_7[11][11]={
        16383, 16383, 12480, 12384, 12384, 12384, 12384, 14560, 8128, 3968, 0,          //  b
        3968, 8128, 14560, 12384, 12384, 12384, 12384, 12384, 6336, 2176, 0,            //  c
        3968, 8128, 14560, 12384, 12384, 12384, 12512, 12480, 16383, 16383, 0,          //  d
        3968, 8128, 15328, 13152, 13152, 13152, 13152, 13152, 5056, 384, 0,             //  e
        192, 192, 16380, 16382, 199, 195, 195, 3, 0, 0, 0,                              //  f
        896, 51136, 52960, 52320, 52320, 52320, 52320, 58976, 32736, 16352, 0,          //  g
        16383, 16383, 192, 96, 96, 96, 224, 16320, 16256, 0, 0,                         //  h
        0, 0, 12288, 12384, 16364, 16364, 12288, 12288, 0, 0, 0,                        //  i
        0, 0, 24576, 57344, 49152, 49248, 65516, 32748, 0, 0, 0,                        //  j
        0, 16383, 16383, 768, 1920, 4032, 7392, 14432, 12288, 0, 0,                     //  k
        0, 0, 12288, 12291, 16383, 16383, 12288, 12288, 0, 0, 0                         //  l
};

const unsigned int16 TEXT_11x16_8[11][11]={
        16352, 16320, 224, 224, 16320, 16320, 224, 224, 16320, 16256, 0,                //  m
        0, 16352, 16352, 96, 96, 96, 96, 224, 16320, 16256, 0,                          //  n
        3968, 8128, 14560, 12384, 12384, 12384, 12384, 14560, 8128, 3968, 0,            //  o
        65504, 65504, 3168, 6240, 6240, 6240, 6240, 7392, 4032, 1920, 0,                //  p
        1920, 4032, 7392, 6240, 6240, 6240, 6240, 3168, 65504, 65504, 0,                //  q
        0, 16352, 16352, 192, 96, 96, 96, 96, 224, 192, 0,                              //  r
        4544, 13280, 13152, 13152, 13152, 13152, 16224, 7744, 0, 0, 0,                  //  s
        96, 96, 8190, 16382, 12384, 12384, 12384, 12288, 0, 0, 0,                       //  t
        4064, 8160, 14336, 12288, 12288, 12288, 12288, 6144, 16352, 16352, 0,           //  u
        96, 480, 1920, 7680, 14336, 14336, 7680, 1920, 480, 96, 0,                      //  v
        2016, 8160, 14336, 7168, 4064, 4064, 7168, 14336, 8160, 2016, 0                 //  w
};

const int16 TEXT_11x16_9[7][11]={
        12384, 14560, 7616, 3968, 1792, 3968, 7616, 14560, 12384, 0, 0,                 //  x
        0, 96, 33248, 59264, 32256, 7680, 1920, 480, 96, 0, 0,                          //  y
        12384, 14432, 15456, 13920, 13152, 12768, 12512, 12384, 12320, 0, 0,            //  z
        0, 128, 448, 8188, 16254, 28679, 24579, 24579, 24579, 0, 0,                     //  {
        0, 0, 0, 0, 16383, 16383, 0, 0, 0, 0, 0,                                        //  |
        0, 24579, 24579, 24579, 28679, 16254, 8188, 448, 128, 0, 0,                     //  }
        16, 24, 12, 4, 12, 24, 16, 24, 12, 4, 0                                         //  ~
};



// Purpose:       Draw a line on a graphic LCD using Bresenham's
//                line drawing algorithm
// Inputs:        (x1, y1) - the start coordinate
//                (x2, y2) - the end coordinate
//                colour - ON or OFF
// Dependencies:  glcd_pixel()
void glcd_line(int x1, int y1, int x2, int y2, int1 colour)
{
   signed int  x, y, addx, addy, dx, dy;
   signed long P;
   int i;
   dx = abs((signed int)(x2 - x1));
   dy = abs((signed int)(y2 - y1));
   x = x1;
   y = y1;

   if(x1 > x2)
      addx = -1;
   else
      addx = 1;
   if(y1 > y2)
      addy = -1;
   else
      addy = 1;

   if(dx >= dy)
   {
      P = 2*dy - dx;

      for(i=0; i<=dx; ++i)
      {
         glcd_pixel(x, y, colour);

         if(P < 0)
         {
            P += 2*dy;
            x += addx;
         }
         else
         {
            P += 2*dy - 2*dx;
            x += addx;
            y += addy;
         }
      }
   }
   else
   {
      P = 2*dx - dy;

      for(i=0; i<=dy; ++i)
      {
         glcd_pixel(x, y, colour);

         if(P < 0)
         {
            P += 2*dx;
            y += addy;
         }
         else
         {
            P += 2*dx - 2*dy;
            x += addx;
            y += addy;
         }
      }
   }
}


// Purpose:       Draw a rectangle on a graphic LCD
// Inputs:        (x1, y1) - the start coordinate
//                (x2, y2) - the end coordinate
//                fill  - YES or NO
//                colour - ON or OFF
// Dependencies:  glcd_pixel(), glcd_line()
void glcd_rect(int x1, int y1, int x2, int y2, int fill, int1 colour)
{
   if(fill)
   {
      int y, ymax;                          // Find the y min and max
      if(y1 < y2)
      {
         y = y1;
         ymax = y2;
      }
      else
      {
         y = y2;
         ymax = y1;
      }

      for(; y<=ymax; ++y)                    // Draw lines to fill the rectangle
         glcd_line(x1, y, x2, y, colour);
   }
   else
   {
      glcd_line(x1, y1, x2, y1, colour);      // Draw the 4 sides
      glcd_line(x1, y2, x2, y2, colour);
      glcd_line(x1, y1, x1, y2, colour);
      glcd_line(x2, y1, x2, y2, colour);
   }
}


// Purpose:       Draw a bar (wide line) on a graphic LCD
// Inputs:        (x1, y1) - the start coordinate
//                (x2, y2) - the end coordinate
//                width  - The number of pixels wide
//                colour - ON or OFF
void glcd_bar(int x1, int y1, int x2, int y2, int width, int1 colour)
{
   signed int  x, y, addx, addy, j;
   signed long P, dx, dy, c1, c2;
   int i;
   dx = abs((signed int)(x2 - x1));
   dy = abs((signed int)(y2 - y1));
   x = x1;
   y = y1;
   c1 = -dx*x1 - dy*y1;
   c2 = -dx*x2 - dy*y2;

   if(x1 > x2)
   {
      addx = -1;
      c1 = -dx*x2 - dy*y2;
      c2 = -dx*x1 - dy*y1;
   }
   else
      addx = 1;
   if(y1 > y2)
   {
      addy = -1;
      c1 = -dx*x2 - dy*y2;
      c2 = -dx*x1 - dy*y1;
   }
   else
      addy = 1;

   if(dx >= dy)
   {
      P = 2*dy - dx;

      for(i=0; i<=dx; ++i)
      {
         for(j=-(width/2); j<width/2+width%2; ++j)
         {
            if(dx*x+dy*(y+j)+c1 >= 0 && dx*x+dy*(y+j)+c2 <=0)
               glcd_pixel(x, y+j, colour);
         }
         if(P < 0)
         {
            P += 2*dy;
            x += addx;
         }
         else
         {
            P += 2*dy - 2*dx;
            x += addx;
            y += addy;
         }
      }
   }
   else
   {
      P = 2*dx - dy;

      for(i=0; i<=dy; ++i)
      {
         if(P < 0)
         {
            P += 2*dx;
            y += addy;
         }
         else
         {
            P += 2*dx - 2*dy;
            x += addx;
            y += addy;
         }
         for(j=-(width/2); j<width/2+width%2; ++j)
         {
            if(dx*x+dy*(y+j)+c1 >= 0 && dx*x+dy*(y+j)+c2 <=0)
               glcd_pixel(x+j, y, colour);
         }
      }
   }
}


// Purpose:       Draw a circle on a graphic LCD
// Inputs:        (x,y) - the center of the circle
//                radius - the radius of the circle
//                fill - YES or NO
//                colour - ON or OFF
void glcd_circle(int x, int y, int radius, int1 fill, int1 colour)
{
   signed int a, b, P;
   a = 0;
   b = radius;
   P = 1 - radius;

   do
   {
      if(fill)
      {
         glcd_line(x-a, y+b, x+a, y+b, colour);
         glcd_line(x-a, y-b, x+a, y-b, colour);
         glcd_line(x-b, y+a, x+b, y+a, colour);
         glcd_line(x-b, y-a, x+b, y-a, colour);
      }
      else
      {
         glcd_pixel(a+x, b+y, colour);
         glcd_pixel(b+x, a+y, colour);
         glcd_pixel(x-a, b+y, colour);
         glcd_pixel(x-b, a+y, colour);
         glcd_pixel(b+x, y-a, colour);
         glcd_pixel(a+x, y-b, colour);
         glcd_pixel(x-a, y-b, colour);
         glcd_pixel(x-b, y-a, colour);
      }

      if(P < 0)
         P+= 3 + 2*a++;
      else
         P+= 5 + 2*(a++ - b--);
    } while(a <= b);
}


// Purpose:       Write a char on a graphic LCD using the 3x6 bit font
void glcd_putc36(char c)
{
    int j, k, l, m;                     // Loop counters
    BYTE pixelData[3];                     // Stores character data
	
	if(c == '\r') { glcd_x = 0; return; }
	if(c == '\n') { glcd_y += 7 * glcd_size; return; }

    if(c < 0x20 || c > 0x60) // Checks if the letter is in the first text array
       memcpy(pixelData, TEXT_3x6_1[0], 3);   // Default to space
    else
       memcpy(pixelData, TEXT_3x6_1[c-' '], 3);
       
    for(j=0; j<3; ++j, glcd_x+=glcd_size) {        // Loop through character byte data
       for(k=2; k<8*glcd_size; ++k) {         // Loop through the vertical pixels
          if(bit_test(pixelData[j], k)) {// Check if the pixel should be set
             for(l=0; l<glcd_size; ++l) {     // The next two loops change the
                for(m=0; m<glcd_size; ++m){
                   glcd_pixel(glcd_x+m, glcd_y+k*glcd_size+l, glcd_colour); // Draws the pixel
                }
             }
          }
       }
    }
  glcd_x += glcd_size;
}



// Purpose:       Write text string in 3x6 font on a graphic LCD
void glcd_text36(char* textptr)
{
    for(; *textptr != '\0'; ++textptr) 					// Loop through the passed string
		glcd_putc36(*textptr);
}




// Purpose:       Write a char on a graphic LCD using the 5x7 bit font
// Inputs:        (x,y) - The upper left coordinate of the first letter
//                textptr - A pointer to an array of text to display
//                size - The size of the text: 1 = 5x7, 2 = 10x14, ...
//                colour - ON or OFF
void glcd_putc57(char c)
{
    int j, k, l, m;                     // Loop counters
    BYTE pixelData[5];                     // Stores character data
	
	if(c == '\r') { glcd_x = 0; return; }
	if(c == '\n') { glcd_y += 8 * glcd_size; return; }

    if(c < 'S') // Checks if the letter is in the first text array
       memcpy(pixelData, TEXT_5x7_1[c-' '], 5);
    else if(c <= '~') // Check if the letter is in the second array
       memcpy(pixelData, TEXT_5x7_2[c-'S'], 5);
    else
       memcpy(pixelData, TEXT_5x7_1[0], 5);   // Default to space
       
    for(j=0; j<5; ++j, glcd_x+=glcd_size) {        // Loop through character byte data
       for(k=0; k<7*glcd_size; ++k) {         // Loop through the vertical pixels
          if(bit_test(pixelData[j], k)) {// Check if the pixel should be set
             for(l=0; l<glcd_size; ++l) {     // The next two loops change the character's size
                for(m=0; m<glcd_size; ++m) {
                   glcd_pixel(glcd_x+m, glcd_y+k*glcd_size+l, glcd_colour); // Draws the pixel
                }
             }
          }
       }
    }
   glcd_x += glcd_size;
}



// Purpose:       Write text string in 5x7 font on a graphic LCD
void glcd_text57(char* textptr)
{
    for(; *textptr != '\0'; ++textptr) 					// Loop through the passed string
		glcd_putc57(*textptr);
}



// Purpose:       Write a char in 8x12 font on a graphic LCD
void glcd_putc812(char c)
{
	int j, k, l, m;                     // Loop counters
	BYTE pixelData[12];                 // Stores character data
	
	if(c == '\r') { glcd_x = 0; return; }
	if(c == '\n') { glcd_y += 12 * glcd_size; return; }

    if(c < '5') // Checks if the letter is in the first text array
       memcpy(pixelData, TEXT_8x12_1[c-' '], 12);
    else if(c < 'J') // Check if the letter is in the second array
       memcpy(pixelData, TEXT_8x12_2[c-'5'], 12);
    else if(c < '_') // Check if the letter is in the third array
       memcpy(pixelData, TEXT_8x12_3[c-'J'], 12);
    else if(c < 't') // Check if the letter is in the forth array
       memcpy(pixelData, TEXT_8x12_4[c-'_'], 12);
    else if(c <= '~') // Check if the letter is in the fifth array
       memcpy(pixelData, TEXT_8x12_5[c-'t'], 12);
    else
       memcpy(pixelData, TEXT_8x12_1[0], 12);   // Default to space

    for(j=0; j<12; ++j) {        // Loop through character byte data
       for(k=0; k<8*glcd_size; ++k) {          // Loop through the horizontal pixels
           if(bit_test(pixelData[j], k)) { // Check if the pixel should be set
             for(l=0; l<glcd_size; ++l) {       // The next two loops change the character's size
                for(m=0; m<glcd_size; ++m) {
                   glcd_pixel(glcd_x+((7-k)*glcd_size)+l, glcd_y+m+j*glcd_size, glcd_colour); // Draws the pixel
                }
             }
          }
       }
    }
    glcd_x += 8 * glcd_size;
}



// Purpose:       Write text string in 8x12 font on a graphic LCD
void glcd_text812(char* textptr)
{
    for(; *textptr != '\0'; ++textptr) 					// Loop through the passed string
		glcd_putc812(*textptr);
}




// Purpose:       Write a char in 9x16 font on a graphic LCD
void glcd_putc916(char c)
{
	byte j, k, l, m;                     // Loop counters
	unsigned int16 pixelData[9];                 // Stores character data
	
	if(c == '\r') { glcd_x = 0; return; }
	if(c == '\n') { glcd_y += 16 * glcd_size; return; }

	if(c < ' ' || c > '~')
       memcpy(pixelData, TEXT_9x16_1[0], 18);  // Non printable defaults to a space
    else if(c < '.') 							// Check if the letter is in the third array
       memcpy(pixelData, TEXT_9x16_1[c-' '], 18);
    else if(c < '<') 							// Checks if the letter is in the first text array
       memcpy(pixelData, TEXT_9x16_2[c-'.'], 18);
    else if(c < 'J') 							// Check if the letter is in the second array
       memcpy(pixelData, TEXT_9x16_3[c-'<'], 18);
    else if(c < 'X') 							// Check if the letter is in the third array
       memcpy(pixelData, TEXT_9x16_4[c-'J'], 18);
    else if(c < 'f') 							// Check if the letter is in the third array
       memcpy(pixelData, TEXT_9x16_5[c-'X'], 18);
    else if(c < 't') 							// Check if the letter is in the third array
       memcpy(pixelData, TEXT_9x16_6[c-'f'], 18);
    else										// Must be in the third array
       memcpy(pixelData, TEXT_9x16_7[c-'t'], 18);
       
    for(j=0; j<9; ++j, glcd_x+=glcd_size) {        	// Loop through character byte data
       for(k=0; k<16*glcd_size; ++k) {         		// Loop through the vertical pixels
          if(bit_test(pixelData[j], k)) {			// Check if the pixel should be set
             for(l=0; l<glcd_size; ++l) {     		// The next two loops change the character's size
                for(m=0; m<glcd_size; ++m) {
                   glcd_pixel(glcd_x+m, glcd_y+k*glcd_size+l, glcd_colour); // Draw the pixel
                }
             }
          }
       }
    }
}



// Purpose:       Write text string in 9x16 font on a graphic LCD
void glcd_text916(char* textptr)
{
    for(; *textptr != '\0'; ++textptr) 					// Loop through the passed string
		glcd_putc916(*textptr);
}





// Purpose:       Write a char in 11x16 font on a graphic LCD
void glcd_putc1116(char c)
{
	byte j, k, l, m;                     // Loop counters
	unsigned int16 pixelData[11];                 // Stores character data
	
	if(c == '\r') { glcd_x = 0; return; }
	if(c == '\n') { glcd_y += 16 * glcd_size; return; }

	if(c < ' ' || c > '~')
       memcpy(pixelData, TEXT_11x16_1[0], 22);  	// Non printable defaults to a space
    else if(c < ' ' + 11) 							// Checks if the letter is in the first text array
       memcpy(pixelData, TEXT_11x16_1[c-' '], 22);
    else if(c < '+' + 11) 							// Checks if the letter is in the next text array
       memcpy(pixelData, TEXT_11x16_2[c-'+'], 22);
    else if(c < '6' + 11) 							// Checks if the letter is in the next text array
       memcpy(pixelData, TEXT_11x16_3[c-'6'], 22);
    else if(c < 'A' + 11) 							// Checks if the letter is in the next text array
       memcpy(pixelData, TEXT_11x16_4[c-'A'], 22);
    else if(c < 'L' + 11) 							// Checks if the letter is in the next text array
       memcpy(pixelData, TEXT_11x16_5[c-'L'], 22);
    else if(c < 'W' + 11) 							// Checks if the letter is in the next text array
       memcpy(pixelData, TEXT_11x16_6[c-'W'], 22);
    else if(c < 'b' + 11) 							// Checks if the letter is in the next text array
       memcpy(pixelData, TEXT_11x16_7[c-'b'], 22);
    else if(c < 'm' + 11) 							// Checks if the letter is in the next text array
       memcpy(pixelData, TEXT_11x16_8[c-'m'], 22);
    else											// Must be in the last array
       memcpy(pixelData, TEXT_11x16_9[c-'x'], 22);
       
    for(j=0; j<11; ++j, glcd_x+=glcd_size) {        // Loop through character byte data
       for(k=0; k<16*glcd_size; ++k) {         		// Loop through the vertical pixels
          if(bit_test(pixelData[j], k)) {			// Check if the pixel should be set
             for(l=0; l<glcd_size; ++l) {     		// The next two loops change the character's size
                for(m=0; m<glcd_size; ++m) {
                   glcd_pixel(glcd_x+m, glcd_y+k*glcd_size+l, glcd_colour); // Draw the pixel
                }
             }
          }
       }
    }
}



// Purpose:       Write text string in 11x16 font on a graphic LCD
void glcd_text1116(char* textptr)
{
    for(; *textptr != '\0'; ++textptr) 					// Loop through the passed string
		glcd_putc1116(*textptr);
}

